@@ -3,9 +3,12 @@ from os.path import basename
 import psycopg2
 from django.contrib.gis.geos import GEOSGeometry
 from django.db import transaction
+from django.utils.translation import ugettext as _
 from psycopg2 import sql
 from rest_framework.serializers import (
+    BooleanField,
     CharField,
+    ChoiceField,
     IntegerField,
     ModelSerializer,
     SerializerMethodField,
@@ -15,6 +18,7 @@ from rest_framework.serializers import (
 
 from .models import (
     CommandSource,
+    CSVSource,
     Field,
     GeoJSONSource,
     GeometryTypes,
@@ -263,3 +267,106 @@ class WMTSSourceSerialize(SourceSerializer):
     class Meta:
         model = WMTSSource
         fields = "__all__"
+
+
+class CSVSourceSerializer(FileSourceSerializer):
+    coordinate_reference_system = CharField(required=True)
+    encoding = CharField(required=True)
+    field_separator = CharField(required=True)
+    decimal_separator = CharField(required=True)
+    char_delimiter = CharField(required=True)
+    coordinates_field = CharField(required=True)
+    number_lines_to_ignore = IntegerField(required=True)
+    use_header = BooleanField(required=True)
+
+    latitude_field = CharField(required=False)
+    longitude_field = CharField(required=False)
+    latlong_field = CharField(required=False)
+    coordinates_field_count = CharField(required=False)
+    coordinates_separator = CharField(required=False)
+    geom_type = ChoiceField(
+        default=GeometryTypes.Point.value, choices=GeometryTypes.choices()
+    )
+
+    class Meta:
+        model = CSVSource
+        fields = "__all__"
+        extra_kwargs = {
+            "file": {"write_only": True},
+            "settings": {"write_only": True},
+        }
+
+    def to_internal_value(self, data):
+        validated_data = super().to_internal_value(data)
+        validated_data["settings"] = {
+            "coordinate_reference_system": validated_data.pop(
+                "coordinate_reference_system"
+            ),
+            "encoding": validated_data.pop("encoding"),
+            "field_separator": validated_data.pop("field_separator"),
+            "decimal_separator": validated_data.pop("decimal_separator"),
+            "char_delimiter": validated_data.pop("char_delimiter"),
+            "coordinates_field": validated_data.get("coordinates_field"),
+            "number_lines_to_ignore": validated_data.pop("number_lines_to_ignore"),
+            "use_header": validated_data.pop("use_header"),
+        }
+        if validated_data.get("coordinates_field") == "one_column":
+            validated_data["settings"].update(
+                {
+                    "latlong_field": validated_data.pop("latlong_field"),
+                    "coordinates_field_count": validated_data.pop(
+                        "coordinates_field_count"
+                    ),
+                    "coordinates_separator": validated_data.pop(
+                        "coordinates_separator"
+                    ),
+                }
+            )
+
+        elif validated_data.get("coordinates_field") == "two_columns":
+            validated_data["settings"].update(
+                {
+                    "latitude_field": validated_data.pop("latitude_field"),
+                    "longitude_field": validated_data.pop("longitude_field"),
+                }
+            )
+        validated_data.pop("coordinates_field")
+        return validated_data
+
+    def validate(self, data):
+        validated_data = super().validate(data)
+        if data["settings"]["coordinates_field"] == "one_column":
+            if not data["settings"].get("latlong_field"):
+                raise ValidationError(
+                    _(
+                        "latlong_field must be defined when coordinates are set to one column"
+                    )
+                )
+            if not data["settings"].get("coordinates_field_count"):
+                raise ValidationError(
+                    _(
+                        "Coordinates order must be specified when coordinates are set to one column"
+                    )
+                )
+            if not data["settings"].get("coordinates_separator"):
+                raise ValidationError(
+                    _(
+                        "Coordinates separator must be specified when coordinates are set to one column"
+                    )
+                )
+        elif data["settings"]["coordinates_field"] == "two_columns":
+            if not data["settings"].get("latitude_field"):
+                raise ValidationError(
+                    _(
+                        "Latitude field must be specified when coordinates are set to two columns"
+                    )
+                )
+            if not data["settings"].get("longitude_field"):
+                raise ValidationError(
+                    _(
+                        "Longitude field must be specified when coordinates are set to two columns"
+                    )
+                )
+        else:
+            raise ValidationError(_("Incorrect value for coordinates field"))
+        return validated_data
